@@ -118,11 +118,7 @@ class ReceivingBatchViews(APIBatchView):
         return {'purchases': purchases}
 
     def normalize_eligible_purchase(self, purchase):
-        return {
-            'key': purchase.uuid,
-            'department_uuid': purchase.department_uuid,
-            'display': self.render_eligible_purchase(purchase),
-        }
+        return self.handler.normalize_eligible_purchase(purchase)
 
     def render_eligible_purchase(self, purchase):
         return self.handler.render_eligible_purchase(purchase)
