@@ -343,8 +343,9 @@ class Form(object):
                  model_instance=None, model_class=None, appstruct=UNSPECIFIED, nodes={}, enums={}, labels={},
                  assume_local_times=False, renderers=None,
                  hidden={}, widgets={}, defaults={}, validators={}, required={}, helptext={}, focus_spec=None,
-                 action_url=None, cancel_url=None, use_buefy=None, component='tailbone-form'):
-
+                 action_url=None, cancel_url=None, use_buefy=None, component='tailbone-form',
+                 vuejs_field_converters={},
+    ):
         self.fields = None
         if fields is not None:
             self.set_fields(fields)
@@ -380,6 +381,7 @@ class Form(object):
         self.cancel_url = cancel_url
         self.use_buefy = use_buefy
         self.component = component
+        self.vuejs_field_converters = vuejs_field_converters or {}
 
     @property
     def component_studly(self):
@@ -702,6 +704,9 @@ class Form(object):
         """
         return self.helptext[key]
 
+    def set_vuejs_field_converter(self, field, converter):
+        self.vuejs_field_converters[field] = converter
+
     def render(self, template=None, **kwargs):
         if not template:
             if self.readonly and not self.use_buefy:
@@ -788,6 +793,11 @@ class Form(object):
         model value for the given field.  This JS will be written as part of
         the overall response, to be interpreted on the client side.
         """
+        if field.name in self.vuejs_field_converters:
+            convert = self.vuejs_field_converters[field.name]
+            value = convert(field.cstruct)
+            return json.dumps(value)
+
         if isinstance(field.schema.typ, deform.FileData):
             # TODO: we used to always/only return 'null' here but hopefully
             # this also works, to show existing filename when present
@@ -807,6 +817,16 @@ class Form(object):
         except Exception as error:
             raise TailboneJSONFieldError(field.name, error)
 
+    def get_error_messages(self, field):
+        if field.error:
+            return field.error.messages()
+
+        error = self.make_deform_form().error
+        if error:
+            if isinstance(error, colander.Invalid):
+                if error.node.name == field.name:
+                    return error.messages()
+
     def messages_json(self, messages):
         dump = json.dumps(messages)
         dump = dump.replace("'", '&apos;')
@@ -828,7 +848,10 @@ class Form(object):
             return ''
 
         # TODO: fair bit of duplication here, should merge with deform.mako
-        label = HTML.tag('label', self.get_label(field_name), for_=field_name)
+        label = kwargs.get('label')
+        if not label:
+            label = self.get_label(field_name)
+        label = HTML.tag('label', label, for_=field_name)
         field = self.render_field_value(field_name) or ''
         field_div = HTML.tag('div', class_='field', c=[field])
         contents = [label, field_div]
