@@ -1,8 +1,124 @@
-// -*- coding: utf-8 -*-
+
 /**********************************************************************
  * jQuery UI plugins for Tailbone
  **********************************************************************/
 
+/**********************************************************************
+ * gridcore plugin
+ **********************************************************************/
+
+(function($) {
+
+    $.widget('tailbone.gridcore', {
+
+        _create: function() {
+
+            var that = this;
+
+            // Add hover highlight effect to grid rows during mouse-over.
+            // this.element.on('mouseenter', 'tbody tr:not(.header)', function() {
+            this.element.on('mouseenter', 'tr:not(.header)', function() {
+                $(this).addClass('hovering');
+            });
+            // this.element.on('mouseleave', 'tbody tr:not(.header)', function() {
+            this.element.on('mouseleave', 'tr:not(.header)', function() {
+                $(this).removeClass('hovering');
+            });
+
+            // do some extra stuff for grids with checkboxes
+
+            // mark rows selected on page load, as needed
+            this.element.find('tr:not(.header) td.checkbox :checkbox:checked').each(function() {
+                $(this).parents('tr:first').addClass('selected');
+            });
+
+            // (un-)check all rows when clicking check-all box in header
+            if (this.element.find('tr.header td.checkbox :checkbox').length) {
+                this.element.on('click', 'tr.header td.checkbox :checkbox', function() {
+                    var checked = $(this).prop('checked');
+                    var rows = that.element.find('tr:not(.header)');
+                    rows.find('td.checkbox :checkbox').prop('checked', checked);
+                    if (checked) {
+                        rows.addClass('selected');
+                    } else {
+                        rows.removeClass('selected');
+                    }
+                    that.element.trigger('gridchecked', that.count_selected());
+                });
+            }
+
+            // when row with checkbox is clicked, toggle selected status,
+            // unless clicking checkbox (since that already toggles it) or a
+            // link (since that does something completely different)
+            this.element.on('click', 'tr:not(.header)', function(event) {
+                var el = $(event.target);
+                if (!el.is('a') && !el.is(':checkbox')) {
+                    $(this).find('td.checkbox :checkbox').click();
+                }
+            });
+
+            this.element.on('change', 'tr:not(.header) td.checkbox :checkbox', function() {
+                if (this.checked) {
+                    $(this).parents('tr:first').addClass('selected');
+                } else {
+                    $(this).parents('tr:first').removeClass('selected');
+                }
+                that.element.trigger('gridchecked', that.count_selected());
+            });
+
+            // Show 'more' actions when user hovers over 'more' link.
+            this.element.on('mouseenter', '.actions a.more', function() {
+                that.element.find('.actions div.more').hide();
+                $(this).siblings('div.more')
+                    .show()
+                    .position({my: 'left-5 top-4', at: 'left top', of: $(this)});
+            });
+            this.element.on('mouseleave', '.actions div.more', function() {
+                $(this).hide();
+            });
+
+            // Add speed bump for "Delete Row" action, if grid is so configured.
+            if (this.element.data('delete-speedbump')) {
+                this.element.on('click', 'tr:not(.header) .actions a.delete', function() {
+                    return confirm("Are you sure you wish to delete this object?");
+                });
+            }
+        },
+
+        count_selected: function() {
+            return this.element.find('tr:not(.header) td.checkbox :checkbox:checked').length;
+        },
+
+        // TODO: deprecate / remove this?
+        count_checked: function() {
+            return this.count_selected();
+        },
+
+        selected_rows: function() {
+            return this.element.find('tr:not(.header) td.checkbox :checkbox:checked').parents('tr:first');
+        },
+
+        all_uuids: function() {
+            var uuids = [];
+            this.element.find('tr:not(.header)').each(function() {
+                uuids.push($(this).data('uuid'));
+            });
+            return uuids;
+        },
+
+        selected_uuids: function() {
+            var uuids = [];
+            this.element.find('tr:not(.header) td.checkbox :checkbox:checked').each(function() {
+                uuids.push($(this).parents('tr:first').data('uuid'));
+            });
+            return uuids;
+        }
+
+    });
+
+})( jQuery );
+
+
 /**********************************************************************
  * gridwrapper plugin
  **********************************************************************/
@@ -23,7 +139,10 @@
             this.default_filters = this.filters.find('#default-filters');
             this.clear_filters = this.filters.find('#clear-filters');
             this.save_defaults = this.filters.find('#save-defaults');
-            this.grid = this.element.find('.newgrid');
+            this.grid = this.element.find('.grid');
+
+            // add standard grid behavior
+            this.grid.gridcore();
 
             // Enhance filters etc.
             this.filters.find('.filter').gridfilter();
@@ -58,6 +177,10 @@
                 }
             });
 
+            this.add_filter.on('selectmenuopen', function(event, ui) {
+                show_all_options($(this));
+            });
+
             // Intercept filters form submittal, and submit via AJAX instead.
             this.filters_form.on('submit', function() {
                 var settings = {filter: true, partial: true};
@@ -117,11 +240,11 @@
             });
 
             // Refresh data when user clicks a sortable column header.
-            this.element.on('click', 'thead th.sortable a', function() {
-                var th = $(this).parent();
+            this.element.on('click', 'tr.header a', function() {
+                var td = $(this).parent();
                 var data = {
-                    sortkey: th.data('sortkey'),
-                    sortdir: (th.hasClass('sorted') && th.hasClass('asc')) ? 'desc' : 'asc',
+                    sortkey: $(this).data('sortkey'),
+                    sortdir: (td.hasClass('asc')) ? 'desc' : 'asc',
                     page: 1,
                     partial: true
                 };
@@ -143,48 +266,6 @@
                 that.refresh(this.search.substring(1)); // remove leading '?'
                 return false;
             });
-
-            // Add hover highlight effect to grid rows during mouse-over.
-            this.element.on('mouseenter', 'tbody tr', function() {
-                $(this).addClass('hovering');
-            });
-            this.element.on('mouseleave', 'tbody tr', function() {
-                $(this).removeClass('hovering');
-            });
-
-            // Do some extra stuff for grids with checkboxes.
-            if (this.grid.hasClass('selectable')) {
-
-                // (Un-)Check all rows when clicking check-all box in header.
-                this.element.on('click', 'thead th.checkbox input', function() {
-                    var checked = $(this).prop('checked');
-                    that.grid.find('tbody td.checkbox input').prop('checked', checked);
-                });
-
-                // Select current row when clicked, unless clicking checkbox
-                // (since that already does select the row) or a link (since
-                // that does something completely different).
-                this.element.on('click', 'tbody td.checkbox input', function(event) {
-                    event.stopPropagation();
-                });
-                this.element.on('click', 'tbody a', function(event) {
-                    event.stopPropagation();
-                });
-                this.element.on('click', 'tbody tr', function() {
-                    $(this).find('td.checkbox input').click();
-                });
-            }
-
-            // Show 'more' actions when user hovers over 'more' link.
-            this.element.on('mouseenter', '.actions a.more', function() {
-                that.grid.find('.actions div.more').hide();
-                $(this).siblings('div.more')
-                    .show()
-                    .position({my: 'left-5 top-4', at: 'left top', of: $(this)});
-            });
-            this.element.on('mouseleave', '.actions div.more', function() {
-                $(this).hide();
-            });
         },
 
         // Refreshes the visible data within the grid, according to the given settings.
@@ -193,9 +274,30 @@
             this.element.mask("Refreshing data...");
             $.get(this.grid.data('url'), settings, function(data) {
                 that.grid.replaceWith(data);
-                that.grid = that.element.find('.newgrid');
+                that.grid = that.element.find('.grid');
+                that.grid.gridcore();
                 that.element.unmask();
             });
+        },
+
+        results_count: function(as_text) {
+            var count = null;
+            var match = /showing \d+ thru \d+ of (\S+)/.exec(this.element.find('.pager .showing').text());
+            if (match) {
+                count = match[1];
+                if (!as_text) {
+                    count = parseInt(count, 10);
+                }
+            }
+            return count;
+        },
+
+        all_uuids: function() {
+            return this.grid.gridcore('all_uuids');
+        },
+
+        selected_uuids: function() {
+            return this.grid.gridcore('selected_uuids');
         }
 
     });
@@ -219,7 +321,7 @@
             this.checkbox = this.element.find('input[name$="-active"]');
             this.label = this.element.find('label');
             this.inputs = this.element.find('.inputs');
-            this.add_filter = this.element.parents('.newgrid-wrapper').find('#add-filter');
+            this.add_filter = this.element.parents('.grid-wrapper').find('#add-filter');
 
             // Hide the checkbox and label, and add button for toggling active status.
             this.checkbox.addClass('ui-helper-hidden-accessible');
@@ -250,8 +352,12 @@
                 }
             });
 
+            this.verb_select.on('selectmenuopen', function(event, ui) {
+                show_all_options($(this));
+            });
+
             // Enhance any date values with datepicker widget.
-            this.inputs.find('.value input[type="date"]').datepicker({
+            this.inputs.find('.value input[data-datepicker="true"]').datepicker({
                 dateFormat: 'yy-mm-dd',
                 changeYear: true,
                 changeMonth: true
@@ -259,9 +365,14 @@
 
             // Enhance any choice/dropdown values with selectmenu.
             this.inputs.find('.value select').selectmenu({
+                // provide sane width for value dropdown
                 width: '15em'
             });
 
+            this.inputs.find('.value select').on('selectmenuopen', function(event, ui) {
+                show_all_options($(this));
+            });
+
             // Listen for button click, to keep checkbox in sync.
             this._on(this.activebutton, {
                 click: function(e) {
