@@ -10,20 +10,17 @@
 
 $(function() {
     
-    $('.newgrid-wrapper').gridwrapper();
-
     $('#execute-batch').click(function() {
         if (has_execution_options) {
             $('#execution-options-dialog').dialog({
                 title: "Execution Options",
-                width: 500,
-                height: 300,
+                width: 600,
                 modal: true,
                 buttons: [
                     {
                         text: "Execute",
                         click: function(event) {
-                            $(event.target).button('option', 'label', "Executing, please wait...").button('disable');
+                            dialog_button(event).button('option', 'label', "Executing, please wait...").button('disable');
                             $('form[name="batch-execution"]').submit();
                         }
                     },
