@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2021 Lance Edgar
+#  Copyright © 2010-2022 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -89,15 +89,8 @@ def new_request(event):
     if rattail_config:
         app = rattail_config.get_app()
         auth = app.get_auth_handler()
-        request.tailbone_cached_permissions = auth.cache_permissions(
+        request.tailbone_cached_permissions = auth.get_permissions(
             Session(), request.user)
-    # TODO: until we know otherwise, let's assume this is not needed
-    # else:
-    #     # TODO: not sure why this would really work, or even be
-    #     # needed, if there was no rattail config?
-    #     from rattail.db.auth import cache_permissions
-    #     request.tailbone_cached_permissions = cache_permissions(
-    #         Session(), request.user)
 
 
 def before_render(event):
@@ -108,6 +101,7 @@ def before_render(event):
     request = event.get('request') or threadlocal.get_current_request()
 
     renderer_globals = event
+    renderer_globals['rattail_app'] = request.rattail_config.get_app()
     renderer_globals['h'] = helpers
     renderer_globals['url'] = request.route_url
     renderer_globals['rattail'] = rattail
@@ -157,8 +151,18 @@ def before_render(event):
             renderer_globals['background_color'] = request.rattail_config.get(
                 'tailbone', 'background_color')
 
-        # maybe set custom stylesheet for Buefy themes
+        # buefy themes get some extra treatment
         if should_use_buefy(request):
+
+            # declare vue.js and buefy versions to use.  the default
+            # values here are "quite conservative" as of this writing,
+            # perhaps too much so, but at least they should work fine.
+            renderer_globals['vue_version'] = request.rattail_config.get(
+                'tailbone', 'vue_version') or '2.6.10'
+            renderer_globals['buefy_version'] = request.rattail_config.get(
+                'tailbone', 'buefy_version') or '0.8.6'
+
+            # maybe set custom stylesheet
             css = None
             if request.user:
                 css = request.rattail_config.get('tailbone.{}'.format(request.user.uuid),
