@@ -50,9 +50,12 @@ class UserLogin(colander.MappingSchema):
 
 @colander.deferred
 def current_password_correct(node, kw):
+    request = kw['request']
+    app = request.rattail_config.get_app()
+    auth = app.get_auth_handler()
     user = kw['user']
     def validate(node, value):
-        if not authenticate_user(Session(), user.username, value):
+        if not auth.authenticate_user(Session(), user.username, value):
             raise colander.Invalid(node, "The password is incorrect")
     return validate
 
@@ -135,7 +138,9 @@ class AuthenticationView(View):
         return context
 
     def authenticate_user(self, username, password):
-        return authenticate_user(Session(), username, password)
+        app = self.get_rattail_app()
+        auth = app.get_auth_handler()
+        return auth.authenticate_user(Session(), username, password)
 
     def logout(self, **kwargs):
         """
@@ -173,7 +178,7 @@ class AuthenticationView(View):
             return self.redirect(self.request.get_referrer())
 
         use_buefy = self.get_use_buefy()
-        schema = ChangePassword().bind(user=self.request.user)
+        schema = ChangePassword().bind(user=self.request.user, request=self.request)
         form = forms.Form(schema=schema, request=self.request, use_buefy=use_buefy)
         if form.validate(newstyle=True):
             set_user_password(self.request.user, form.validated['new_password'])
