@@ -83,6 +83,8 @@ class BatchMasterView(MasterView):
     has_worksheet = False
     has_worksheet_file = False
 
+    input_file_template_config_section = 'rattail.batch'
+
     grid_columns = [
         'id',
         'description',
@@ -115,7 +117,9 @@ class BatchMasterView(MasterView):
 
     def __init__(self, request):
         super(BatchMasterView, self).__init__(request)
-        self.handler = self.get_handler()
+        self.batch_handler = self.get_handler()
+        # TODO: deprecate / remove this (?)
+        self.handler = self.batch_handler
 
     @classmethod
     def get_handler_factory(cls, rattail_config):
@@ -155,6 +159,10 @@ class BatchMasterView(MasterView):
         factory = self.get_handler_factory(self.rattail_config)
         return factory(self.rattail_config)
 
+    @property
+    def input_file_template_config_prefix(self):
+        return '{}.input_file_template'.format(self.batch_handler.batch_key)
+
     def download_path(self, batch, filename):
         return self.rattail_config.batch_filepath(batch.batch_key, batch.uuid, filename)
 
@@ -615,15 +623,10 @@ class BatchMasterView(MasterView):
     def get_row_status_enum(self):
         return self.model_row_class.STATUS
 
-    def render_upc(self, row, field):
-        upc = row.upc
-        if not upc:
-            return ""
-        text = upc.pretty()
-        if row.product_uuid:
-            url = self.request.route_url('products.view', uuid=row.product_uuid)
-            return tags.link_to(text, url)
-        return text
+    def render_upc_pretty(self, row, field):
+        upc = getattr(row, field)
+        if upc:
+            return upc.pretty()
 
     def render_row_status(self, row, column):
         code = row.status_code
@@ -660,6 +663,10 @@ class BatchMasterView(MasterView):
         # sequence
         f.set_readonly('sequence')
 
+        # upc (default rendering, just in case there is such a field
+        # on our row model)
+        f.set_renderer('upc', self.render_upc_pretty)
+
         # status_code
         if self.model_row_class:
             f.set_enum('status_code', self.model_row_class.STATUS)
@@ -784,28 +791,30 @@ class BatchMasterView(MasterView):
         route_prefix = self.get_route_prefix()
         use_buefy = self.get_use_buefy()
 
+        schema = None
         if self.has_execution_options(batch):
             if batch is None:
                 batch = self.model_class
             schema = self.make_execute_schema(batch)
-            for field in schema:
+            if schema:
+                for field in schema:
 
-                # if field does not yet have a default, maybe provide one from session storage
-                if field.default is colander.null:
-                    key = 'batch.{}.execute_option.{}'.format(batch.batch_key, field.name)
-                    if key in self.request.session:
-                        defaults[field.name] = self.request.session[key]
+                    # if field does not yet have a default, maybe provide one from session storage
+                    if field.default is colander.null:
+                        key = 'batch.{}.execute_option.{}'.format(batch.batch_key, field.name)
+                        if key in self.request.session:
+                            defaults[field.name] = self.request.session[key]
 
-                # make sure field label is preserved
-                if field.title:
-                    labels = kwargs.setdefault('labels', {})
-                    labels[field.name] = field.title
+                    # make sure field label is preserved
+                    if field.title:
+                        labels = kwargs.setdefault('labels', {})
+                        labels[field.name] = field.title
 
-                # auto-convert select widgets for buefy theme
-                if use_buefy and isinstance(field.widget, forms.widgets.PlainSelectWidget):
-                    field.widget = dfwidget.SelectWidget(values=field.widget.values)
+                    # auto-convert select widgets for buefy theme
+                    if use_buefy and isinstance(field.widget, forms.widgets.PlainSelectWidget):
+                        field.widget = dfwidget.SelectWidget(values=field.widget.values)
 
-        else:
+        if not schema:
             schema = colander.Schema()
 
         kwargs['use_buefy'] = use_buefy
@@ -850,7 +859,7 @@ class BatchMasterView(MasterView):
 
             # launch thread to invoke handler action
             thread = Thread(target=self.action_subprocess_thread,
-                            args=(batch.uuid, port, username, batch_action, progress),
+                            args=((batch.uuid,), port, username, batch_action, progress),
                             kwargs=kwargs)
             thread.start()
 
@@ -859,7 +868,7 @@ class BatchMasterView(MasterView):
 
             # launch thread to populate batch; that will update session progress directly
             target = getattr(self, '{}_thread'.format(batch_action))
-            thread = Thread(target=target, args=(batch.uuid, user_uuid, progress), kwargs=kwargs)
+            thread = Thread(target=target, args=((batch.uuid,), user_uuid, progress), kwargs=kwargs)
             thread.start()
 
         return self.render_progress(progress, {
@@ -894,7 +903,7 @@ class BatchMasterView(MasterView):
         log.debug("launching command in subprocess: %s", cmd)
         subprocess.check_call(cmd)
 
-    def action_subprocess_thread(self, batch_uuid, port, username, handler_action, progress, **kwargs):
+    def action_subprocess_thread(self, key, port, username, handler_action, progress, **kwargs):
         """
         This method is sort of an alternative thread target for batch actions,
         to be used in the event versioning is enabled for the main process but
@@ -902,6 +911,8 @@ class BatchMasterView(MasterView):
         launch a separate process with versioning disabled in order to act on
         the batch.
         """
+        batch_uuid = key[0]
+
         # figure out the (sub)command args we'll be passing
         subargs = [
             '--batch-type',
@@ -1147,18 +1158,27 @@ class BatchMasterView(MasterView):
         """
         Batch rows are editable only until batch is complete or executed.
         """
+        if not (self.rows_editable or self.rows_editable_but_not_directly):
+            return False
+
         batch = self.get_parent(row)
-        return self.rows_editable and not batch.executed and not batch.complete
+        if batch.complete or batch.executed:
+            return False
+
+        return True
 
     def row_deletable(self, row):
         """
         Batch rows are deletable only until batch is complete or executed.
         """
-        if self.rows_deletable:
-            batch = self.get_parent(row)
-            if not batch.executed and not batch.complete:
-                return True
-        return False
+        if not self.rows_deletable:
+            return False
+
+        batch = self.get_parent(row)
+        if batch.complete or batch.executed:
+            return False
+
+        return True
 
     def template_kwargs_view_row(self, **kwargs):
         kwargs['batch_model_title'] = kwargs['parent_model_title']
@@ -1216,7 +1236,7 @@ class BatchMasterView(MasterView):
     def execute_error_message(self, error):
         return "Batch execution failed: {}".format(simple_error(error))
 
-    def execute_thread(self, batch_uuid, user_uuid, progress, **kwargs):
+    def execute_thread(self, key, user_uuid, progress, **kwargs):
         """
         Thread target for executing a batch with progress indicator.
         """
@@ -1224,7 +1244,7 @@ class BatchMasterView(MasterView):
         # session here; can't use tailbone because it has web request
         # transaction binding etc.
         session = RattailSession()
-        batch = session.query(self.model_class).get(batch_uuid)
+        batch = self.get_instance_for_key(key, session)
         user = session.query(model.User).get(user_uuid)
         try:
             result = self.handler.do_execute(batch, user=user, progress=progress, **kwargs)
