@@ -172,7 +172,7 @@ class VendorInvoiceView(FileBatchMasterView):
         return kwargs
 
     def init_batch(self, batch):
-        parser = require_invoice_parser(batch.parser_key)
+        parser = require_invoice_parser(self.rattail_config, batch.parser_key)
         vendor = api.get_vendor(self.Session(), parser.vendor_key)
         if not vendor:
             self.request.session.flash("No vendor setting found in database for key: {}".format(parser.vendor_key))
