@@ -500,6 +500,9 @@ class PendingCustomerView(MasterView):
         super(PendingCustomerView, self).configure_grid(g)
 
         g.set_enum('status_code', self.enum.PENDING_CUSTOMER_STATUS)
+        g.filters['status_code'].default_active = True
+        g.filters['status_code'].default_verb = 'not_equal'
+        g.filters['status_code'].default_value = six.text_type(self.enum.PENDING_CUSTOMER_STATUS_RESOLVED)
 
         g.set_sort_defaults('display_name')
         g.set_link('id')
@@ -510,7 +513,63 @@ class PendingCustomerView(MasterView):
 
         f.set_enum('status_code', self.enum.PENDING_CUSTOMER_STATUS)
 
-        f.set_renderer('user', self.render_user)
+        # created
+        if self.creating:
+            f.remove('created')
+        else:
+            f.set_readonly('created')
+
+        # user
+        if self.creating:
+            f.remove('user')
+        else:
+            f.set_readonly('user')
+            f.set_renderer('user', self.render_user)
+
+    def editable_instance(self, pending):
+        if pending.status_code == self.enum.PENDING_CUSTOMER_STATUS_RESOLVED:
+            return False
+        return True
+
+    def resolve_person(self):
+        model = self.model
+        pending = self.get_instance()
+        redirect = self.redirect(self.get_action_url('view', pending))
+
+        uuid = self.request.POST['person_uuid']
+        person = self.Session.query(model.Person).get(uuid)
+        if not person:
+            self.request.session.flash("Person not found!", 'error')
+            return redirect
+
+        app = self.get_rattail_app()
+        people_handler = app.get_people_handler()
+        people_handler.resolve_person(pending, person, self.request.user)
+        self.Session.flush()
+        return redirect
+
+    @classmethod
+    def defaults(cls, config):
+        cls._defaults(config)
+        cls._pending_customer_defaults(config)
+
+    @classmethod
+    def _pending_customer_defaults(cls, config):
+        route_prefix = cls.get_route_prefix()
+        instance_url_prefix = cls.get_instance_url_prefix()
+        permission_prefix = cls.get_permission_prefix()
+        model_title = cls.get_model_title()
+
+        # resolve person
+        config.add_tailbone_permission(permission_prefix,
+                                       '{}.resolve_person'.format(permission_prefix),
+                                       "Resolve a {} as a Person".format(model_title))
+        config.add_route('{}.resolve_person'.format(route_prefix),
+                         '{}/resolve-person'.format(instance_url_prefix),
+                         request_method='POST')
+        config.add_view(cls, attr='resolve_person',
+                        route_name='{}.resolve_person'.format(route_prefix),
+                        permission='{}.resolve_person'.format(permission_prefix))
 
 
 # # TODO: this is referenced by some custom apps, but should be moved??
