@@ -36,8 +36,7 @@ from deform import widget as dfwidget
 from webhelpers2.html import tags, HTML
 
 from tailbone import grids
-from tailbone.db import Session
-from tailbone.views import MasterView, AutocompleteView
+from tailbone.views import MasterView
 
 
 class EmployeeView(MasterView):
@@ -46,6 +45,8 @@ class EmployeeView(MasterView):
     """
     model_class = model.Employee
     has_versions = True
+    touchable = True
+    supports_autocomplete = True
 
     labels = {
         'id': "ID",
@@ -82,6 +83,7 @@ class EmployeeView(MasterView):
     def configure_grid(self, g):
         super(EmployeeView, self).configure_grid(g)
         route_prefix = self.get_route_prefix()
+        use_buefy = self.get_use_buefy()
 
         # phone
         g.set_joiner('phone', lambda q: q.outerjoin(model.EmployeePhoneNumber, sa.and_(
@@ -113,21 +115,23 @@ class EmployeeView(MasterView):
             g.hide_column('username')
 
         # id
-        if self.request.has_perm('{}.edit'.format(route_prefix)):
+        if self.has_perm('edit'):
             g.set_link('id')
         else:
-            g.hide_column('id')
+            g.remove('id')
             del g.filters['id']
 
         # status
-        if self.request.has_perm('{}.edit'.format(route_prefix)):
+        if self.has_perm('view_all'):
             g.set_enum('status', self.enum.EMPLOYEE_STATUS)
             g.filters['status'].default_active = True
             g.filters['status'].default_verb = 'equal'
-            # TODO: why must we set unicode string value here?
-            g.filters['status'].default_value = six.text_type(self.enum.EMPLOYEE_STATUS_CURRENT)
+            if use_buefy:
+                g.filters['status'].default_value = six.text_type(self.enum.EMPLOYEE_STATUS_CURRENT)
+            else:
+                g.filters['status'].default_value = self.enum.EMPLOYEE_STATUS_CURRENT
         else:
-            g.hide_column('status')
+            g.remove('status')
             del g.filters['status']
 
         g.filters['first_name'].default_active = True
@@ -150,7 +154,7 @@ class EmployeeView(MasterView):
 
     def query(self, session):
         q = session.query(model.Employee).join(model.Person)
-        if not self.request.has_perm('employees.edit'):
+        if not self.has_perm('view_all'):
             q = q.filter(model.Employee.status == self.enum.EMPLOYEE_STATUS_CURRENT)
         return q
 
@@ -185,7 +189,12 @@ class EmployeeView(MasterView):
         employee = f.model_instance
 
         f.set_renderer('person', self.render_person)
-        f.set_renderer('users', self.render_users)
+
+        if self.creating or self.editing:
+            f.remove('users')
+        else:
+            f.set_readonly('users')
+            f.set_renderer('users', self.render_users)
 
         f.set_renderer('stores', self.render_stores)
         f.set_label('stores', "Stores") # TODO: should not be necessary
@@ -304,31 +313,21 @@ class EmployeeView(MasterView):
             (model.EmployeeDepartment, 'employee_uuid'),
         ]
 
-# TODO: deprecate / remove this
-EmployeesView = EmployeeView
+    @classmethod
+    def defaults(cls, config):
+        cls._defaults(config)
+        cls._employee_defaults(config)
 
+    @classmethod
+    def _employee_defaults(cls, config):
+        permission_prefix = cls.get_permission_prefix()
+        model_title_plural = cls.get_model_title_plural()
 
-class EmployeesAutocomplete(AutocompleteView):
-    """
-    Autocomplete view for the Employee model, but restricted to return only
-    results for current employees.
-    """
-    mapped_class = model.Person
-    fieldname = 'display_name'
-
-    def filter_query(self, q):
-        return q.join(model.Employee)\
-            .filter(model.Employee.status == self.enum.EMPLOYEE_STATUS_CURRENT)
-
-    def value(self, person):
-        return person.employee.uuid
+        # view *all* employees
+        config.add_tailbone_permission(permission_prefix,
+                                       '{}.view_all'.format(permission_prefix),
+                                       "View *all* (not just current) {}".format(model_title_plural))
 
 
 def includeme(config):
-
-    # autocomplete
-    config.add_route('employees.autocomplete',  '/employees/autocomplete')
-    config.add_view(EmployeesAutocomplete, route_name='employees.autocomplete',
-                    renderer='json', permission='employees.list')
-
     EmployeeView.defaults(config)
