@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2018 Lance Edgar
+#  Copyright © 2010-2021 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -33,7 +33,7 @@ import colander
 from tailbone.views import MasterView
 
 
-class InventoryAdjustmentReasonsView(MasterView):
+class InventoryAdjustmentReasonView(MasterView):
     """
     Master view for inventory adjustment reasons.
     """
@@ -48,11 +48,11 @@ class InventoryAdjustmentReasonsView(MasterView):
     ]
 
     def configure_grid(self, g):
-        super(InventoryAdjustmentReasonsView, self).configure_grid(g)
+        super(InventoryAdjustmentReasonView, self).configure_grid(g)
         g.set_sort_defaults('code')
 
     def configure_form(self, f):
-        super(InventoryAdjustmentReasonsView, self).configure_form(f)
+        super(InventoryAdjustmentReasonView, self).configure_form(f)
 
         # code
         f.set_validator('code', self.unique_code)
@@ -66,6 +66,9 @@ class InventoryAdjustmentReasonsView(MasterView):
         if query.count():
             raise colander.Invalid(node, "Code must be unique")
 
+# TODO: deprecate / remove this
+InventoryAdjustmentReasonsView = InventoryAdjustmentReasonView
+
 
 def includeme(config):
-    InventoryAdjustmentReasonsView.defaults(config)
+    InventoryAdjustmentReasonView.defaults(config)
