@@ -85,6 +85,7 @@ class ProductView(MasterView):
     has_versions = True
     results_downloadable_xlsx = True
     supports_autocomplete = True
+    configurable = True
 
     labels = {
         'item_id': "Item ID",
@@ -663,6 +664,8 @@ class ProductView(MasterView):
         classes = []
         if product.not_for_sale:
             classes.append('not-for-sale')
+        if product.discontinued:
+            classes.append('discontinued')
         if product.deleted:
             classes.append('deleted')
         if classes:
@@ -1093,6 +1096,7 @@ class ProductView(MasterView):
         """
         AJAX view for fetching various types of price history for a product.
         """
+        app = self.get_rattail_app()
         product = self.get_instance()
 
         typ = self.request.params.get('type', 'regular')
@@ -1106,8 +1110,9 @@ class ProductView(MasterView):
         for history in data:
             history = dict(history)
             price = history['price']
-            history['price'] = float(price)
-            history['price_display'] = "${:0.2f}".format(price)
+            if price is not None:
+                history['price'] = float(price)
+                history['price_display'] = app.render_currency(price)
             changed = localtime(self.rattail_config, history['changed'], from_utc=True)
             history['changed'] = six.text_type(changed)
             history['changed_display_html'] = raw_datetime(self.rattail_config, changed)
@@ -1148,6 +1153,7 @@ class ProductView(MasterView):
         use_buefy = self.get_use_buefy()
 
         kwargs['image_url'] = self.handler.get_image_url(product)
+        kwargs['product_key_field'] = self.rattail_config.product_key()
 
         # add price history, if user has access
         if self.rattail_config.versioning_enabled() and self.has_perm('versions'):
@@ -1904,6 +1910,26 @@ class ProductView(MasterView):
         if batch.batch_key == 'delproduct':
             return self.request.route_url('batch.delproduct.view', uuid=batch.uuid)
 
+    def configure_get_simple_settings(self):
+        return [
+
+            # key field
+            {'section': 'rattail',
+             'option': 'product.key'},
+            {'section': 'rattail',
+             'option': 'product.key_title'},
+
+            # handling
+            {'section': 'rattail',
+             'option': 'products.convert_type2_for_gpc_lookup',
+             'type': bool},
+
+            # display
+            {'section': 'tailbone',
+             'option': 'products.show_pod_image',
+             'type': bool},
+        ]
+
     @classmethod
     def defaults(cls, config):
         cls._product_defaults(config)
@@ -1966,6 +1992,7 @@ class PendingProductView(MasterView):
     model_class = model.PendingProduct
     route_prefix = 'pending_products'
     url_prefix = '/products/pending'
+    bulk_deletable = True
 
     labels = {
         'regular_price_amount': "Regular Price",
@@ -1975,10 +2002,10 @@ class PendingProductView(MasterView):
 
     grid_columns = [
         '_product_key_',
-        'department_name',
         'brand_name',
         'description',
         'size',
+        'department_name',
         'created',
         'user',
         'status_code',
@@ -1986,12 +2013,15 @@ class PendingProductView(MasterView):
 
     form_fields = [
         '_product_key_',
-        'department_name',
-        'department',
         'brand_name',
         'brand',
         'description',
         'size',
+        'department_name',
+        'department',
+        'vendor_name',
+        'vendor',
+        'unit_cost',
         'case_size',
         'regular_price_amount',
         'special_order',
@@ -2004,14 +2034,10 @@ class PendingProductView(MasterView):
     def configure_grid(self, g):
         super(PendingProductView, self).configure_grid(g)
 
-        # product key
-        if '_product_key_' in g.columns:
-            key = self.rattail_config.product_key()
-            g.replace('_product_key_', key)
-            g.set_label(key, self.rattail_config.product_key_title(key))
-            g.set_link(key)
-
         g.set_enum('status_code', self.enum.PENDING_PRODUCT_STATUS)
+        g.filters['status_code'].default_active = True
+        g.filters['status_code'].default_verb = 'not_equal'
+        g.filters['status_code'].default_value = six.text_type(self.enum.PENDING_PRODUCT_STATUS_RESOLVED)
 
         g.set_sort_defaults('created', 'desc')
 
@@ -2022,13 +2048,6 @@ class PendingProductView(MasterView):
         model = self.model
         pending = f.model_instance
 
-        # product key
-        if '_product_key_' in f:
-            key = self.rattail_config.product_key()
-            f.replace('_product_key_', key)
-            f.set_label(key, self.rattail_config.product_key_title(key))
-            f.set_renderer(key, self.render_product_key_value)
-
         # department
         if self.creating or self.editing:
             if 'department' in f:
@@ -2063,10 +2082,35 @@ class PendingProductView(MasterView):
             f.set_renderer('brand', self.render_brand)
             if pending.brand:
                 f.remove('brand_name')
+            elif pending.brand_name:
+                f.remove('brand')
 
         # description
         f.set_required('description')
 
+        # vendor
+        if self.creating or self.editing:
+            if 'vendor' in f:
+                f.remove('vendor_name')
+                f.replace('vendor', 'vendor_uuid')
+                f.set_node('vendor_uuid', colander.String())
+                vendor_display = ""
+                if self.request.method == 'POST':
+                    if self.request.POST.get('vendor_uuid'):
+                        vendor = self.Session.query(model.Vendor).get(self.request.POST['vendor_uuid'])
+                        if vendor:
+                            vendor_display = six.text_type(vendor)
+                f.set_widget('vendor_uuid', forms.widgets.JQueryAutocompleteWidget(
+                    field_display=vendor_display,
+                    service_url=self.request.route_url('vendors.autocomplete')))
+                f.set_label('vendor_uuid', "Vendor")
+        else:
+            f.set_renderer('vendor', self.render_vendor)
+            if pending.vendor:
+                f.remove('vendor_name')
+            elif pending.vendor_name:
+                f.remove('vendor')
+
         # case_size
         f.set_type('case_size', 'quantity')
 
@@ -2096,6 +2140,11 @@ class PendingProductView(MasterView):
             # f.set_readonly('status_code')
             f.set_enum('status_code', self.enum.PENDING_PRODUCT_STATUS)
 
+    def editable_instance(self, pending):
+        if pending.status_code == self.enum.PENDING_PRODUCT_STATUS_RESOLVED:
+            return False
+        return True
+
     def objectify(self, form, data=None):
         if data is None:
             data = form.validated
@@ -2117,6 +2166,69 @@ class PendingProductView(MasterView):
 
         return pending
 
+    def before_delete(self, pending):
+        """
+        Event hook, called just before deletion is attempted.
+        """
+        model = self.model
+        model_title = self.get_model_title()
+        count = self.Session.query(model.CustomerOrderItem)\
+                            .filter(model.CustomerOrderItem.pending_product == pending)\
+                            .count()
+        if count:
+            self.request.session.flash("Cannot delete this {} because it is still "
+                                       "referenced by {} Customer Orders.".format(model_title, count),
+                                       'error')
+            return self.redirect(self.get_action_url('view', pending))
+
+        count = self.Session.query(model.CustomerOrderBatchRow)\
+                            .filter(model.CustomerOrderBatchRow.pending_product == pending)\
+                            .count()
+        if count:
+            self.request.session.flash("Cannot delete this {} because it is still "
+                                       "referenced by {} \"new\" Customer Order Batches.".format(model_title, count),
+                                       'error')
+            return self.redirect(self.get_action_url('view', pending))
+
+    def resolve_product(self):
+        model = self.model
+        pending = self.get_instance()
+        redirect = self.redirect(self.get_action_url('view', pending))
+
+        uuid = self.request.POST['product_uuid']
+        product = self.Session.query(model.Product).get(uuid)
+        if not product:
+            self.request.session.flash("Product not found!", 'error')
+            return redirect
+
+        app = self.get_rattail_app()
+        products_handler = app.get_products_handler()
+        products_handler.resolve_product(pending, product, self.request.user)
+        return redirect
+
+    @classmethod
+    def defaults(cls, config):
+        cls._defaults(config)
+        cls._pending_product_defaults(config)
+
+    @classmethod
+    def _pending_product_defaults(cls, config):
+        route_prefix = cls.get_route_prefix()
+        instance_url_prefix = cls.get_instance_url_prefix()
+        permission_prefix = cls.get_permission_prefix()
+        model_title = cls.get_model_title()
+
+        # resolve product
+        config.add_tailbone_permission(permission_prefix,
+                                       '{}.resolve_product'.format(permission_prefix),
+                                       "Resolve a {} as a Product".format(model_title))
+        config.add_route('{}.resolve_product'.format(route_prefix),
+                         '{}/resolve-product'.format(instance_url_prefix),
+                         request_method='POST')
+        config.add_view(cls, attr='resolve_product',
+                        route_name='{}.resolve_product'.format(route_prefix),
+                        permission='{}.resolve_product'.format(permission_prefix))
+
 
 def print_labels(request):
     profile = request.params.get('profile')
