@@ -99,8 +99,10 @@ class PurchasingBatchView(BatchMasterView):
         'upc': "UPC",
         'item_id': "Item ID",
         'brand_name': "Brand",
+        'case_quantity': "Case Size",
         'po_line_number': "PO Line Number",
         'po_unit_cost': "PO Unit Cost",
+        'po_case_size': "PO Case Size",
         'po_total': "PO Total",
     }
 
@@ -129,16 +131,24 @@ class PurchasingBatchView(BatchMasterView):
         'description',
         'size',
         'case_quantity',
+        'ordered',
         'cases_ordered',
         'units_ordered',
+        'received',
         'cases_received',
         'units_received',
+        'damaged',
         'cases_damaged',
         'units_damaged',
+        'expired',
         'cases_expired',
         'units_expired',
+        'mispick',
         'cases_mispick',
         'units_mispick',
+        'missing',
+        'cases_missing',
+        'units_missing',
         'po_line_number',
         'po_unit_cost',
         'po_total',
@@ -637,6 +647,11 @@ class PurchasingBatchView(BatchMasterView):
         g.set_label('catalog_unit_cost', "Catalog Cost")
         g.filters['catalog_unit_cost'].label = "Catalog Unit Cost"
 
+        # po_unit_cost
+        g.set_renderer('po_unit_cost', self.render_row_grid_cost)
+        g.set_label('po_unit_cost', "PO Cost")
+        g.filters['po_unit_cost'].label = "PO Unit Cost"
+
         # invoice_unit_cost
         g.set_renderer('invoice_unit_cost', self.render_row_grid_cost)
         g.set_label('invoice_unit_cost', "Invoice Cost")
@@ -694,7 +709,17 @@ class PurchasingBatchView(BatchMasterView):
         f.set_readonly('case_quantity')
 
         # quantity fields
+        f.set_renderer('ordered', self.render_row_quantity)
+        f.set_renderer('shipped', self.render_row_quantity)
+        f.set_renderer('received', self.render_row_quantity)
+        f.set_renderer('damaged', self.render_row_quantity)
+        f.set_renderer('expired', self.render_row_quantity)
+        f.set_renderer('mispick', self.render_row_quantity)
+        f.set_renderer('missing', self.render_row_quantity)
+
         f.set_type('case_quantity', 'quantity')
+        f.set_type('po_case_size', 'quantity')
+        f.set_type('invoice_case_size', 'quantity')
         f.set_type('cases_ordered', 'quantity')
         f.set_type('units_ordered', 'quantity')
         f.set_type('cases_shipped', 'quantity')
@@ -707,6 +732,8 @@ class PurchasingBatchView(BatchMasterView):
         f.set_type('units_expired', 'quantity')
         f.set_type('cases_mispick', 'quantity')
         f.set_type('units_mispick', 'quantity')
+        f.set_type('cases_missing', 'quantity')
+        f.set_type('units_missing', 'quantity')
 
         # currency fields
         # nb. we only show "total" fields as currency, but not case or
@@ -729,7 +756,8 @@ class PurchasingBatchView(BatchMasterView):
 
         # credits
         f.set_readonly('credits')
-        f.set_renderer('credits', self.render_row_credits)
+        if self.viewing:
+            f.set_renderer('credits', self.render_row_credits)
 
         if self.creating:
             f.remove_fields(
@@ -765,28 +793,62 @@ class PurchasingBatchView(BatchMasterView):
             else:
                 f.remove_field('product')
 
-    def render_row_credits(self, row, field):
-        if not row.credits:
-            return ""
+    def render_row_quantity(self, row, field):
+        app = self.get_rattail_app()
+        cases = getattr(row, 'cases_{}'.format(field))
+        units = getattr(row, 'units_{}'.format(field))
+        return app.render_cases_units(cases, units)
 
+    def make_row_credits_grid(self, row):
+        use_buefy = self.get_use_buefy()
         route_prefix = self.get_route_prefix()
-        columns = [
-            'credit_type',
-            'cases_shorted',
-            'units_shorted',
-            'credit_total',
-        ]
-        g = grids.Grid(
+        factory = self.get_grid_factory()
+
+        g = factory(
             key='{}.row_credits'.format(route_prefix),
-            data=row.credits,
-            columns=columns,
-            labels={'credit_type': "Type",
-                    'cases_shorted': "Cases",
-                    'units_shorted': "Units"})
+            data=[] if use_buefy else row.credits,
+            columns=[
+                'credit_type',
+                # 'cases_shorted',
+                # 'units_shorted',
+                'shorted',
+                'credit_total',
+                'expiration_date',
+                # 'mispick_upc',
+                # 'mispick_brand_name',
+                # 'mispick_description',
+                # 'mispick_size',
+            ],
+            labels={
+                'credit_type': "Type",
+                'cases_shorted': "Cases",
+                'units_shorted': "Units",
+                'shorted': "Quantity",
+                'credit_total': "Total",
+                'mispick_upc': "Mispick UPC",
+                'mispick_brand_name': "MP Brand",
+                'mispick_description': "MP Description",
+                'mispick_size': "MP Size",
+            })
+
         g.set_type('cases_shorted', 'quantity')
         g.set_type('units_shorted', 'quantity')
         g.set_type('credit_total', 'currency')
-        return HTML.literal(g.render_grid())
+
+        return g
+
+    def render_row_credits(self, row, field):
+        use_buefy = self.get_use_buefy()
+        if not use_buefy and not row.credits:
+            return
+
+        g = self.make_row_credits_grid(row)
+
+        if use_buefy:
+            return HTML.literal(
+                g.render_buefy_table_element(data_prop='rowData.credits'))
+        else:
+            return HTML.literal(g.render_grid())
 
 #     def item_lookup(self, value, field=None):
 #         """
