@@ -428,8 +428,11 @@ class OrderingBatchView(PurchasingBatchView):
                 self.handler.add_row(batch, row)
 
             # update row quantities
-            self.handler.update_row_quantity(row, cases_ordered=cases_ordered,
-                                             units_ordered=units_ordered)
+            try:
+                self.handler.update_row_quantity(row, cases_ordered=cases_ordered,
+                                                 units_ordered=units_ordered)
+            except Exception as error:
+                return {'error': six.text_type(error)}
 
         else: # empty order quantities
 
