@@ -51,6 +51,21 @@ from tailbone.views.purchasing import PurchasingBatchView
 
 log = logging.getLogger(__name__)
 
+POSSIBLE_RECEIVING_MODES = [
+    'received',
+    'damaged',
+    'expired',
+    # 'mispick',
+    'missing',
+]
+
+POSSIBLE_CREDIT_TYPES = [
+    'damaged',
+    'expired',
+    # 'mispick',
+    'missing',
+]
+
 
 class ReceivingBatchView(PurchasingBatchView):
     """
@@ -63,7 +78,12 @@ class ReceivingBatchView(PurchasingBatchView):
     index_title = "Receiving"
     downloadable = True
     bulk_deletable = True
-    rows_editable = True
+    configurable = True
+    config_title = "Receiving"
+
+    rows_editable = False
+    rows_editable_but_not_directly = True
+    rows_deletable = True
 
     default_uom_is_case = True
 
@@ -144,6 +164,7 @@ class ReceivingBatchView(PurchasingBatchView):
         'cases_received',
         'units_received',
         'catalog_unit_cost',
+        'po_unit_cost',
         'invoice_unit_cost',
         'invoice_total_calculated',
         'credits',
@@ -152,6 +173,7 @@ class ReceivingBatchView(PurchasingBatchView):
     ]
 
     row_form_fields = [
+        'sequence',
         'item_entry',
         'upc',
         'item_id',
@@ -161,24 +183,36 @@ class ReceivingBatchView(PurchasingBatchView):
         'description',
         'size',
         'case_quantity',
+        'ordered',
         'cases_ordered',
         'units_ordered',
+        'shipped',
         'cases_shipped',
         'units_shipped',
+        'received',
         'cases_received',
         'units_received',
+        'damaged',
         'cases_damaged',
         'units_damaged',
+        'expired',
         'cases_expired',
         'units_expired',
+        'mispick',
         'cases_mispick',
         'units_mispick',
+        'missing',
+        'cases_missing',
+        'units_missing',
+        'catalog_unit_cost',
         'po_line_number',
         'po_unit_cost',
+        'po_case_size',
         'po_total',
         'invoice_line_number',
         'invoice_unit_cost',
         'invoice_cost_confirmed',
+        'invoice_case_size',
         'invoice_total',
         'invoice_total_calculated',
         'status_code',
@@ -291,6 +325,8 @@ class ReceivingBatchView(PurchasingBatchView):
         else:
             form.set_widget('workflow',
                             forms.widgets.JQuerySelectWidget(values=values))
+        if len(workflows) == 1:
+            form.set_default('workflow', workflows[0]['workflow_key'])
 
         form.submit_label = "Continue"
         form.cancel_url = self.get_index_url()
@@ -311,17 +347,14 @@ class ReceivingBatchView(PurchasingBatchView):
         return self.render_to_response('create', context)
 
     def row_deletable(self, row):
+
+        # first run it through the normal logic, if that doesn't like
+        # it then we won't either
+        if not super(ReceivingBatchView, self).row_deletable(row):
+            return False
+
         batch = row.batch
 
-        # don't allow if master view has disabled that entirely
-        if not self.rows_deletable:
-            return False
-
-        # can never delete rows for complete/executed batches
-        # TODO: not so sure about the 'complete' part though..?
-        if batch.executed or batch.complete:
-            return False
-
         # can always delete rows from truck dump parent
         if batch.is_truck_dump_parent():
             return True
@@ -351,7 +384,7 @@ class ReceivingBatchView(PurchasingBatchView):
         super(ReceivingBatchView, self).configure_form(f)
         model = self.model
         batch = f.model_instance
-        allow_truck_dump = self.handler.allow_truck_dump_receiving()
+        allow_truck_dump = self.batch_handler.allow_truck_dump_receiving()
         workflow = self.request.matchdict.get('workflow_key')
         route_prefix = self.get_route_prefix()
         use_buefy = self.get_use_buefy()
@@ -461,9 +494,9 @@ class ReceivingBatchView(PurchasingBatchView):
             and self.purchase_order_fieldname == 'purchase'):
             if use_buefy:
                 f.replace('purchase', 'purchase_uuid')
-                purchases = self.handler.get_eligible_purchases(
+                purchases = self.batch_handler.get_eligible_purchases(
                     vendor, self.enum.PURCHASE_BATCH_MODE_RECEIVING)
-                values = [(p.uuid, self.handler.render_eligible_purchase(p))
+                values = [(p.uuid, self.batch_handler.render_eligible_purchase(p))
                           for p in purchases]
                 f.set_widget('purchase_uuid', dfwidget.SelectWidget(values=values))
                 f.set_label('purchase_uuid', "Purchase Order")
@@ -485,6 +518,13 @@ class ReceivingBatchView(PurchasingBatchView):
         if self.creating:
             f.remove('invoice_total_calculated')
 
+        # hide all invoice fields if batch does not have invoice file
+        if not self.creating and not self.batch_handler.has_invoice_file(batch):
+            f.remove('invoice_file',
+                     'invoice_date',
+                     'invoice_number',
+                     'invoice_total')
+
         # receiving_complete
         if self.creating:
             f.remove('receiving_complete')
@@ -498,19 +538,30 @@ class ReceivingBatchView(PurchasingBatchView):
                          'invoice_parser_key')
 
             elif workflow == 'from_invoice':
-                f.remove('truck_dump_batch_uuid')
                 f.set_required('invoice_file')
                 f.set_required('invoice_parser_key')
+                f.remove('truck_dump_batch_uuid',
+                         'po_number',
+                         'invoice_date',
+                         'invoice_number')
 
             elif workflow == 'from_po':
                 f.remove('truck_dump_batch_uuid',
+                         'date_ordered',
+                         'po_number',
                          'invoice_file',
-                         'invoice_parser_key')
+                         'invoice_parser_key',
+                         'invoice_date',
+                         'invoice_number')
 
             elif workflow == 'from_po_with_invoice':
-                f.remove('truck_dump_batch_uuid')
                 f.set_required('invoice_file')
                 f.set_required('invoice_parser_key')
+                f.remove('truck_dump_batch_uuid',
+                         'date_ordered',
+                         'po_number',
+                         'invoice_date',
+                         'invoice_number')
 
             elif workflow == 'truck_dump_children_first':
                 f.remove('truck_dump_batch_uuid',
@@ -591,6 +642,95 @@ class ReceivingBatchView(PurchasingBatchView):
             raise NotImplementedError
         return kwargs
 
+    def make_po_vs_invoice_breakdown(self, batch):
+        """
+        Returns a simple breakdown as list of 2-tuples, each of which
+        has the display title as first member, and number of rows as
+        second member.
+        """
+        grouped = {}
+        labels = OrderedDict([
+            ('both', "Found in both PO and Invoice"),
+            ('po_not_invoice', "Found in PO but not Invoice"),
+            ('invoice_not_po', "Found in Invoice but not PO"),
+            ('neither', "Not found in PO nor Invoice"),
+        ])
+
+        for row in batch.active_rows():
+            if row.po_line_number and not row.invoice_line_number:
+                grouped.setdefault('po_not_invoice', []).append(row)
+            elif row.invoice_line_number and not row.po_line_number:
+                grouped.setdefault('invoice_not_po', []).append(row)
+            elif row.po_line_number and row.invoice_line_number:
+                grouped.setdefault('both', []).append(row)
+            else:
+                grouped.setdefault('neither', []).append(row)
+
+        breakdown = []
+
+        for key, label in labels.items():
+            if key in grouped:
+                breakdown.append({
+                    'title': label,
+                    'count': len(grouped[key]),
+                })
+
+        return breakdown
+
+    def template_kwargs_view(self, **kwargs):
+        kwargs = super(ReceivingBatchView, self).template_kwargs_view(**kwargs)
+        batch = kwargs['instance']
+
+        if self.handler.has_purchase_order(batch) and self.handler.has_invoice_file(batch):
+            breakdown = self.make_po_vs_invoice_breakdown(batch)
+
+            factory = self.get_grid_factory()
+            kwargs['po_vs_invoice_breakdown_grid'] = factory(
+                'batch_po_vs_invoice_breakdown',
+                data=breakdown,
+                columns=['title', 'count'])
+
+        return kwargs
+
+    def get_context_credits(self, row):
+        app = self.get_rattail_app()
+        credits_data = []
+        for credit in row.credits:
+            credits_data.append({
+                'uuid': credit.uuid,
+                'credit_type': credit.credit_type,
+                'expiration_date': six.text_type(credit.expiration_date) if credit.expiration_date else None,
+                'cases_shorted': app.render_quantity(credit.cases_shorted),
+                'units_shorted': app.render_quantity(credit.units_shorted),
+                'shorted': app.render_cases_units(credit.cases_shorted,
+                                                  credit.units_shorted),
+                'credit_total': app.render_currency(credit.credit_total),
+                'mispick_upc': '-',
+                'mispick_brand_name': '-',
+                'mispick_description': '-',
+                'mispick_size': '-',
+            })
+        return credits_data
+
+    def template_kwargs_view_row(self, **kwargs):
+        kwargs = super(ReceivingBatchView, self).template_kwargs_view_row(**kwargs)
+        use_buefy = self.get_use_buefy()
+        app = self.get_rattail_app()
+        products_handler = app.get_products_handler()
+        row = kwargs['instance']
+
+        if row.product:
+            kwargs['image_url'] = products_handler.get_image_url(row.product)
+        elif row.upc:
+            kwargs['image_url'] = products_handler.get_image_url(upc=row.upc)
+
+        if use_buefy:
+            kwargs['row_context'] = self.get_context_row(row)
+            kwargs['possible_receiving_modes'] = POSSIBLE_RECEIVING_MODES
+            kwargs['possible_credit_types'] = POSSIBLE_CREDIT_TYPES
+
+        return kwargs
+
     def department_for_purchase(self, purchase):
         pass
 
@@ -734,11 +874,25 @@ class ReceivingBatchView(PurchasingBatchView):
 
     def configure_row_grid(self, g):
         super(ReceivingBatchView, self).configure_row_grid(g)
+        batch = self.get_instance()
 
         # vendor_code
         g.filters['vendor_code'].default_active = True
         g.filters['vendor_code'].default_verb = 'contains'
 
+        # catalog_unit_cost
+        if (self.handler.has_purchase_order(batch)
+            or self.handler.has_invoice_file(batch)):
+            g.remove('catalog_unit_cost')
+
+        # po_unit_cost
+        if self.handler.has_invoice_file(batch):
+            g.remove('po_unit_cost')
+
+        # invoice_unit_cost
+        if not self.handler.has_invoice_file(batch):
+            g.remove('invoice_unit_cost')
+
         # credits
         # note that sorting by credits involves a subquery with group by clause.
         # seems likely there may be a better way? but this seems to work fine
@@ -751,10 +905,9 @@ class ReceivingBatchView(PurchasingBatchView):
 
         # hide 'ordered' columns for truck dump parent, if its "children first"
         # flag is set, since that batch type is only concerned with receiving
-        batch = self.get_instance()
         if batch.is_truck_dump_parent() and not batch.truck_dump_children_first:
-            g.hide_column('cases_ordered')
-            g.hide_column('units_ordered')
+            g.remove('cases_ordered',
+                     'units_ordered')
 
         # add "Transform to Unit" action, if appropriate
         if batch.is_truck_dump_parent():
@@ -771,7 +924,7 @@ class ReceivingBatchView(PurchasingBatchView):
 
         # truck_dump_status
         if not batch.is_truck_dump_parent():
-            g.hide_column('truck_dump_status')
+            g.remove('truck_dump_status')
         else:
             g.set_enum('truck_dump_status', model.PurchaseBatchRow.STATUS)
 
@@ -786,6 +939,13 @@ class ReceivingBatchView(PurchasingBatchView):
 
         return css_class
 
+    def get_row_instance_title(self, row):
+        if row.product:
+            return six.text_type(row.product)
+        if row.upc:
+            return row.upc.pretty()
+        return super(ReceivingBatchView, self).get_row_instance_title(row)
+
     def transform_unit_url(self, row, i):
         # grid action is shown only when we return a URL here
         if self.row_editable(row):
@@ -793,6 +953,36 @@ class ReceivingBatchView(PurchasingBatchView):
                 if row.product and row.product.is_pack_item():
                     return self.get_row_action_url('transform_unit', row)
 
+    def make_row_credits_grid(self, row):
+
+        # first make grid like normal
+        g = super(ReceivingBatchView, self).make_row_credits_grid(row)
+
+        if (self.get_use_buefy()
+            and self.has_perm('edit_row')
+            and self.row_editable(row)):
+
+            # add the Un-Declare action
+            g.main_actions.append(self.make_action(
+                'remove', label="Un-Declare",
+                url='#', icon='trash',
+                link_class='has-text-danger',
+                click_handler='removeCreditInit(props.row)'))
+
+        return g
+
+    def vuejs_convert_quantity(self, cstruct):
+        result = dict(cstruct)
+        if result['cases'] is colander.null:
+            result['cases'] = None
+        elif isinstance(result['cases'], decimal.Decimal):
+            result['cases'] = float(result['cases'])
+        if result['units'] is colander.null:
+            result['units'] = None
+        elif isinstance(result['units'], decimal.Decimal):
+            result['units'] = float(result['units'])
+        return result
+
     def receive_row(self, **kwargs):
         """
         Primary desktop view for row-level receiving.
@@ -804,6 +994,55 @@ class ReceivingBatchView(PurchasingBatchView):
         self.viewing = True
         use_buefy = self.get_use_buefy()
         row = self.get_row_instance()
+
+        # things are a bit different now w/ buefy support..
+        if use_buefy:
+
+            # don't even bother showing this page if that's all the
+            # request was about
+            if self.request.method == 'GET':
+                return self.redirect(self.get_row_action_url('view', row))
+
+            # make sure edit is allowed
+            if not (self.has_perm('edit_row') and self.row_editable(row)):
+                raise self.forbidden()
+
+            # check for JSON POST, which is submitted via AJAX from
+            # the "view row" page
+            if self.request.method == 'POST' and not self.request.POST:
+                data = self.request.json_body
+                kwargs = dict(data)
+
+                # TODO: for some reason quantities can come through as strings?
+                cases = kwargs['quantity']['cases']
+                if cases is not None:
+                    if cases == '':
+                        cases = None
+                    else:
+                        cases = decimal.Decimal(cases)
+                kwargs['cases'] = cases
+                units = kwargs['quantity']['units']
+                if units is not None:
+                    if units == '':
+                        units = None
+                    else:
+                        units = decimal.Decimal(units)
+                kwargs['units'] = units
+                del kwargs['quantity']
+
+                # handler takes care of the receiving logic for us
+                try:
+                    self.batch_handler.receive_row(row, **kwargs)
+
+                except Exception as error:
+                    return self.json_response({'error': six.text_type(error)})
+
+                self.Session.flush()
+                self.Session.refresh(row)
+                return self.json_response({
+                    'ok': True,
+                    'row': self.get_context_row(row)})
+
         batch = row.batch
         permission_prefix = self.get_permission_prefix()
         possible_modes = [
@@ -828,14 +1067,24 @@ class ReceivingBatchView(PurchasingBatchView):
         schema = ReceiveRowForm().bind(session=self.Session())
         form = forms.Form(schema=schema, request=self.request, use_buefy=use_buefy)
         form.cancel_url = self.get_row_action_url('view', row)
+
+        # mode
         mode_values = [(mode, mode) for mode in possible_modes]
         if use_buefy:
-            form.set_widget('mode', dfwidget.SelectWidget(values=mode_values))
+            mode_widget = dfwidget.SelectWidget(values=mode_values)
         else:
-            form.set_widget('mode', forms.widgets.JQuerySelectWidget(values=mode_values))
+            mode_widget = forms.widgets.JQuerySelectWidget(values=mode_values)
+        form.set_widget('mode', mode_widget)
+
+        # quantity
         form.set_widget('quantity', forms.widgets.CasesUnitsWidget(amount_required=True,
                                                                    one_amount_only=True))
+        form.set_vuejs_field_converter('quantity', self.vuejs_convert_quantity)
+
+        # expiration_date
         form.set_type('expiration_date', 'date_jquery')
+
+        # TODO: what is this one about again?
         form.remove_field('quick_receive')
 
         if form.validate(newstyle=True):
@@ -944,12 +1193,61 @@ class ReceivingBatchView(PurchasingBatchView):
         View for declaring a credit, i.e. converting some "received" or similar
         quantity, to a credit of some sort.
         """
+        use_buefy = self.get_use_buefy()
         row = self.get_row_instance()
+
+        # things are a bit different now w/ buefy support..
+        if use_buefy:
+
+            # don't even bother showing this page if that's all the
+            # request was about
+            if self.request.method == 'GET':
+                return self.redirect(self.get_row_action_url('view', row))
+
+            # make sure edit is allowed
+            if not (self.has_perm('edit_row') and self.row_editable(row)):
+                raise self.forbidden()
+
+            # check for JSON POST, which is submitted via AJAX from
+            # the "view row" page
+            if self.request.method == 'POST' and not self.request.POST:
+                data = self.request.json_body
+                kwargs = dict(data)
+
+                # TODO: for some reason quantities can come through as strings?
+                if kwargs['cases'] is not None:
+                    if kwargs['cases'] == '':
+                        kwargs['cases'] = None
+                    else:
+                        kwargs['cases'] = decimal.Decimal(kwargs['cases'])
+                if kwargs['units'] is not None:
+                    if kwargs['units'] == '':
+                        kwargs['units'] = None
+                    else:
+                        kwargs['units'] = decimal.Decimal(kwargs['units'])
+
+                try:
+                    result = self.handler.can_declare_credit(row, **kwargs)
+
+                except Exception as error:
+                    return self.json_response({'error': six.text_type(error)})
+
+                else:
+                    if result:
+                        self.handler.declare_credit(row, **kwargs)
+
+                    else:
+                        return self.json_response({
+                            'error': "Handler says you can't declare that credit; "
+                            "not sure why"})
+
+                self.Session.flush()
+                self.Session.refresh(row)
+                return self.json_response({
+                    'ok': True,
+                    'row': self.get_context_row(row)})
+
         batch = row.batch
-        possible_credit_types = [
-            'damaged',
-            'expired',
-        ]
         context = {
             'row': row,
             'batch': batch,
@@ -963,11 +1261,24 @@ class ReceivingBatchView(PurchasingBatchView):
         }
 
         schema = DeclareCreditForm()
-        form = forms.Form(schema=schema, request=self.request)
-        form.set_widget('credit_type', forms.widgets.JQuerySelectWidget(
-            values=[(m, m) for m in possible_credit_types]))
+        form = forms.Form(schema=schema, request=self.request,
+                          use_buefy=use_buefy)
+        form.cancel_url = self.get_row_action_url('view', row)
+
+        # credit_type
+        values = [(m, m) for m in POSSIBLE_CREDIT_TYPES]
+        if use_buefy:
+            widget = dfwidget.SelectWidget(values=values)
+        else:
+            widget = forms.widgets.JQuerySelectWidget(values=values)
+        form.set_widget('credit_type', widget)
+
+        # quantity
         form.set_widget('quantity', forms.widgets.CasesUnitsWidget(
             amount_required=True, one_amount_only=True))
+        form.set_vuejs_field_converter('quantity', self.vuejs_convert_quantity)
+
+        # expiration_date
         form.set_type('expiration_date', 'date_jquery')
 
         if form.validate(newstyle=True):
@@ -994,6 +1305,54 @@ class ReceivingBatchView(PurchasingBatchView):
         context['parent_title'] = self.get_instance_title(batch)
         return self.render_to_response('declare_credit', context)
 
+    def undeclare_credit(self):
+        """
+        View for un-declaring a credit, i.e. moving the credit amounts
+        back into the "received" tally.
+        """
+        model = self.model
+        row = self.get_row_instance()
+        data = self.request.json_body
+
+        # make sure edit is allowed
+        if not (self.has_perm('edit_row') and self.row_editable(row)):
+            raise self.forbidden()
+
+        # figure out which credit to un-declare
+        credit = None
+        uuid = data.get('uuid')
+        if uuid:
+            credit = self.Session.query(model.PurchaseBatchCredit).get(uuid)
+        if not credit:
+            return {'error': "Credit not found"}
+
+        # un-declare it
+        self.batch_handler.undeclare_credit(row, credit)
+        self.Session.flush()
+        self.Session.refresh(row)
+
+        return {'ok': True,
+                'row': self.get_context_row(row)}
+
+    def get_context_row(self, row):
+        app = self.get_rattail_app()
+        return {
+            'sequence': row.sequence,
+            'case_quantity': float(row.case_quantity) if row.case_quantity is not None else None,
+            'ordered': self.render_row_quantity(row, 'ordered'),
+            'shipped': self.render_row_quantity(row, 'shipped'),
+            'received': self.render_row_quantity(row, 'received'),
+            'cases_received': float(row.cases_received) if row.cases_received is not None else None,
+            'units_received': float(row.units_received) if row.units_received is not None else None,
+            'damaged': self.render_row_quantity(row, 'damaged'),
+            'expired': self.render_row_quantity(row, 'expired'),
+            'mispick': self.render_row_quantity(row, 'mispick'),
+            'missing': self.render_row_quantity(row, 'missing'),
+            'credits': self.get_context_credits(row),
+            'invoice_total_calculated': app.render_currency(row.invoice_total_calculated),
+            'status': row.STATUS[row.status_code],
+        }
+
     def transform_unit_row(self):
         """
         View which transforms the given row, which is assumed to associate with
@@ -1400,6 +1759,24 @@ class ReceivingBatchView(PurchasingBatchView):
         if self.rattail_config.getbool('rattail.batch', 'purchase.mobile_images', default=True):
             return pod.get_image_url(self.rattail_config, row.upc)
 
+    def can_auto_receive(self, batch):
+        if batch.executed:
+            return False
+        if batch.complete:
+            return False
+
+        if batch.is_truck_dump_related():
+            if not batch.is_truck_dump_parent():
+                return False
+            if not batch.truck_dump_children_first():
+                return False
+
+        # only auto-receive once per batch
+        if batch.get_param('auto_received'):
+            return False
+
+        return True
+
     def auto_receive(self):
         """
         View which can "auto-receive" all items in the batch.  Meant only as a
@@ -1452,6 +1829,47 @@ class ReceivingBatchView(PurchasingBatchView):
                 progress.session['success_url'] = success_url
                 progress.session.save()
 
+    def configure_get_simple_settings(self):
+        config = self.rattail_config
+        return [
+
+            # supported workflows
+            {'section': 'rattail.batch',
+             'option': 'purchase.allow_receiving_from_scratch',
+             'type': bool},
+            {'section': 'rattail.batch',
+             'option': 'purchase.allow_receiving_from_invoice',
+             'type': bool},
+            {'section': 'rattail.batch',
+             'option': 'purchase.allow_receiving_from_purchase_order',
+             'type': bool},
+            {'section': 'rattail.batch',
+             'option': 'purchase.allow_receiving_from_purchase_order_with_invoice',
+             'type': bool},
+            {'section': 'rattail.batch',
+             'option': 'purchase.allow_truck_dump_receiving',
+             'type': bool},
+
+            # product handling
+            {'section': 'rattail.batch',
+             'option': 'purchase.allow_cases',
+             'type': bool},
+            {'section': 'rattail.batch',
+             'option': 'purchase.allow_expired_credits',
+             'type': bool},
+
+            # mobile interface
+            {'section': 'rattail.batch',
+             'option': 'purchase.mobile_images',
+             'type': bool},
+            {'section': 'rattail.batch',
+             'option': 'purchase.mobile_quick_receive',
+             'type': bool},
+            {'section': 'rattail.batch',
+             'option': 'purchase.mobile_quick_receive_all',
+             'type': bool},
+        ]
+
     @classmethod
     def defaults(cls, config):
         cls._receiving_defaults(config)
@@ -1466,6 +1884,7 @@ class ReceivingBatchView(PurchasingBatchView):
         url_prefix = cls.get_url_prefix()
         instance_url_prefix = cls.get_instance_url_prefix()
         model_key = cls.get_model_key()
+        model_title = cls.get_model_title()
         permission_prefix = cls.get_permission_prefix()
 
         # new receiving batch using workflow X
@@ -1483,6 +1902,14 @@ class ReceivingBatchView(PurchasingBatchView):
         config.add_view(cls, attr='declare_credit', route_name='{}.declare_credit'.format(route_prefix),
                         permission='{}.edit_row'.format(permission_prefix))
 
+        # un-declare credit
+        config.add_route('{}.undeclare_credit'.format(route_prefix),
+                         '{}/rows/{{row_uuid}}/undeclare-credit'.format(instance_url_prefix))
+        config.add_view(cls, attr='undeclare_credit',
+                        route_name='{}.undeclare_credit'.format(route_prefix),
+                        permission='{}.edit_row'.format(permission_prefix),
+                        renderer='json')
+
         # update row cost
         config.add_route('{}.update_row_cost'.format(route_prefix), '{}/update-row-cost'.format(instance_url_prefix))
         config.add_view(cls, attr='update_row_cost', route_name='{}.update_row_cost'.format(route_prefix),
@@ -1500,11 +1927,13 @@ class ReceivingBatchView(PurchasingBatchView):
                         permission='{}.edit_row'.format(permission_prefix), renderer='json')
 
         # auto-receive all items
-        if not rattail_config.production():
-            config.add_route('{}.auto_receive'.format(route_prefix), '{}/auto-receive'.format(instance_url_prefix),
-                             request_method='POST')
-            config.add_view(cls, attr='auto_receive', route_name='{}.auto_receive'.format(route_prefix),
-                            permission='admin')
+        config.add_tailbone_permission(permission_prefix,
+                                       '{}.auto_receive'.format(permission_prefix),
+                                       "Auto-receive all items for a {}".format(model_title))
+        config.add_route('{}.auto_receive'.format(route_prefix), '{}/auto-receive'.format(instance_url_prefix),
+                         request_method='POST')
+        config.add_view(cls, attr='auto_receive', route_name='{}.auto_receive'.format(route_prefix),
+                        permission='{}.auto_receive'.format(permission_prefix))
 
 
 @colander.deferred
@@ -1537,12 +1966,8 @@ class NewReceivingBatch(colander.Schema):
 class ReceiveRowForm(colander.MappingSchema):
 
     mode = colander.SchemaNode(colander.String(),
-                               validator=colander.OneOf([
-                                   'received',
-                                   'damaged',
-                                   'expired',
-                                   # 'mispick',
-                               ]))
+                               validator=colander.OneOf(
+                                   POSSIBLE_RECEIVING_MODES))
 
     quantity = forms.types.ProductQuantity()
 
@@ -1552,15 +1977,21 @@ class ReceiveRowForm(colander.MappingSchema):
 
     quick_receive = colander.SchemaNode(colander.Boolean())
 
+    def deserialize(self, *args):
+        result = super(ReceiveRowForm, self).deserialize(*args)
+
+        if result['mode'] == 'expired' and not result['expiration_date']:
+            msg = "Expiration date is required for items with 'expired' mode."
+            self.raise_invalid(msg, node=self.get('expiration_date'))
+
+        return result
+
 
 class DeclareCreditForm(colander.MappingSchema):
 
     credit_type = colander.SchemaNode(colander.String(),
-                                      validator=colander.OneOf([
-                                          'damaged',
-                                          'expired',
-                                          # 'mispick',
-                                      ]))
+                                      validator=colander.OneOf(
+                                          POSSIBLE_CREDIT_TYPES))
 
     quantity = forms.types.ProductQuantity()
 
