@@ -43,12 +43,12 @@ import colander
 from deform import widget as dfwidget
 from mako.template import Template
 from pyramid.response import Response
-from webhelpers2.html import HTML
+from webhelpers2.html import HTML, tags
 
 from tailbone import forms
 from tailbone.db import Session
 from tailbone.views import View
-from tailbone.views.exports import ExportMasterView
+from tailbone.views.exports import ExportMasterView, MasterView
 
 
 plu_upc_pattern = re.compile(r'^000000000(\d{5})$')
@@ -213,6 +213,9 @@ class ReportOutputView(ExportMasterView):
     route_prefix = 'report_output'
     url_prefix = '/reports/generated'
     downloadable = True
+    configurable = True
+    config_title = "Reporting"
+    config_url = '/reports/configure'
 
     grid_columns = [
         'id',
@@ -275,18 +278,30 @@ class ReportOutputView(ExportMasterView):
         url = self.get_action_url('download', report)
         return self.render_file_field(path, url=url)
 
-    def template_kwargs_view(self, **kwargs):
-        use_buefy = self.get_use_buefy()
-        if use_buefy:
+    def get_params_context(self, report):
+        params_data = []
+        for name, value in (report.params or {}).items():
+            params_data.append({
+                'key': name,
+                'value': value,
+            })
+        return params_data
 
+    def template_kwargs_view(self, **kwargs):
+        kwargs = super(ReportOutputView, self).template_kwargs_view(**kwargs)
+
+        if self.get_use_buefy():
             report = kwargs['instance']
-            params_data = []
-            for name, value in report.params.items():
-                params_data.append({
-                    'key': name,
-                    'value': value,
-                })
-            kwargs['params_data'] = params_data
+            kwargs['params_data'] = self.get_params_context(report)
+
+        return kwargs
+
+    def template_kwargs_delete(self, **kwargs):
+        kwargs = super(ReportOutputView, self).template_kwargs_delete(**kwargs)
+
+        if self.get_use_buefy():
+            report = kwargs['instance']
+            kwargs['params_data'] = self.get_params_context(report)
 
         return kwargs
 
@@ -295,6 +310,16 @@ class ReportOutputView(ExportMasterView):
         path = report.filepath(self.rattail_config)
         return self.file_response(path)
 
+    def configure_get_simple_settings(self):
+        config = self.rattail_config
+        return [
+
+            # generating
+            {'section': 'tailbone',
+             'option': 'reporting.choosing_uses_form',
+             'type': bool},
+        ]
+
 
 class GenerateReport(View):
     """
@@ -511,6 +536,140 @@ class NewReport(colander.Schema):
                                       validator=valid_report_type)
 
 
+class ProblemReportView(MasterView):
+    """
+    Master view for problem reports
+    """
+    model_title = "Problem Report"
+    model_key = ('system_key', 'problem_key')
+    route_prefix = 'problem_reports'
+    url_prefix = '/reports/problems'
+
+    creatable = False
+    editable = False
+    deletable = False
+    filterable = False
+    pageable = False
+    executable = True
+
+    labels = {
+        'system_key': "System",
+    }
+
+    grid_columns = [
+        'system_key',
+        # 'problem_key',
+        'problem_title',
+        'email_recipients',
+    ]
+
+    form_fields = [
+        'system_key',
+        'problem_title',
+        'email_key',
+        'email_recipients',
+    ]
+
+    def __init__(self, request):
+        super(ProblemReportView, self).__init__(request)
+
+        app = self.get_rattail_app()
+        self.handler = app.get_problem_report_handler()
+
+    def normalize(self, report, keep_report=True):
+        data = {
+            'system_key': report.system_key,
+            'problem_key': report.problem_key,
+            'problem_title': report.problem_title,
+            'email_key': self.handler.get_email_key(report),
+        }
+
+        app = self.get_rattail_app()
+        handler = app.get_mail_handler()
+        email = handler.get_email(data['email_key'])
+        data['email_recipients'] = email.get_recips('all')
+
+        if keep_report:
+            data['_report'] = report
+        return data
+
+    def get_data(self, session=None):
+        data = []
+
+        reports = self.handler.get_all_problem_reports()
+        organized = self.handler.organize_problem_reports(reports)
+
+        for system_key, reports in six.iteritems(organized):
+            for report in six.itervalues(reports):
+                data.append(self.normalize(report))
+
+        return data
+
+    def configure_grid(self, g):
+        super(ProblemReportView, self).configure_grid(g)
+
+        g.set_renderer('email_recipients', self.render_email_recipients)
+
+        g.set_link('problem_key')
+        g.set_link('problem_title')
+
+    def get_instance(self):
+        system_key = self.request.matchdict['system_key']
+        problem_key = self.request.matchdict['problem_key']
+        return self.get_instance_for_key((system_key, problem_key),
+                                         None)
+
+    def get_instance_for_key(self, key, session):
+        report = self.handler.get_problem_report(*key)
+        if report:
+            return self.normalize(report)
+        raise self.notfound()
+
+    def get_instance_title(self, report_info):
+        return report_info['problem_title']
+
+    def make_form_schema(self):
+        return ProblemReportSchema()
+
+    def configure_form(self, f):
+        super(ProblemReportView, self).configure_form(f)
+
+        f.set_renderer('email_key', self.render_email_key)
+        f.set_renderer('email_recipients', self.render_email_recipients)
+
+    def render_email_key(self, report_info, field):
+        email_key = report_info[field]
+        if not email_key:
+            return
+
+        if self.request.has_perm('emailprofiles.view'):
+            text = email_key
+            url = self.request.route_url('emailprofiles.view', key=email_key)
+            return tags.link_to(text, url)
+
+        return email_key
+
+    def render_email_recipients(self, report_info, field):
+        recips = report_info['email_recipients']
+        return ', '.join(recips)
+
+    def execute_instance(self, report_info, user, progress=None, **kwargs):
+        report = report_info['_report']
+        problems = self.handler.run_problem_report(report)
+        return "Report found {} problems".format(len(problems))
+
+
+class ProblemReportSchema(colander.MappingSchema):
+
+    system_key = colander.SchemaNode(colander.String())
+
+    problem_key = colander.SchemaNode(colander.String())
+
+    problem_title = colander.SchemaNode(colander.String())
+
+    email_key = colander.SchemaNode(colander.String())
+
+
 def add_routes(config):
     config.add_route('reports.ordering',        '/reports/ordering')
     config.add_route('reports.inventory',       '/reports/inventory')
@@ -531,3 +690,4 @@ def includeme(config):
     # note that GenerateReport must come first, per route matching
     GenerateReport.defaults(config)
     ReportOutputView.defaults(config)
+    ProblemReportView.defaults(config)
