@@ -33,8 +33,7 @@ from sqlalchemy import orm
 from openpyxl.styles import Font, PatternFill
 
 from rattail.db import model
-from rattail.db.auth import (has_permission, grant_permission, revoke_permission,
-                             administrator_role, guest_role, authenticated_role)
+from rattail.db.auth import administrator_role, guest_role, authenticated_role
 from rattail.excel import ExcelWriter
 
 import colander
@@ -52,10 +51,18 @@ class RoleView(PrincipalMasterView):
     """
     model_class = model.Role
     has_versions = True
+    touchable = True
+
+    labels = {
+        'sync_me': "Sync Attrs & Perms",
+    }
 
     grid_columns = [
         'name',
         'session_timeout',
+        'sync_me',
+        'sync_users',
+        'node_type',
         'notes',
     ]
 
@@ -63,6 +70,10 @@ class RoleView(PrincipalMasterView):
         'name',
         'session_timeout',
         'notes',
+        'sync_me',
+        'sync_users',
+        'node_type',
+        'users',
         'permissions',
     ]
 
@@ -92,6 +103,11 @@ class RoleView(PrincipalMasterView):
         We must prevent edit for certain built-in roles etc., depending on
         current user's permissions.
         """
+        # role with node type specified, can only be edited from a
+        # node of the same type
+        if role.node_type and role.node_type != self.rattail_config.node_type():
+            return False
+
         # only "root" can edit Administrator
         if role is administrator_role(self.Session()):
             return self.request.is_root
@@ -115,6 +131,11 @@ class RoleView(PrincipalMasterView):
         """
         We must prevent deletion for all built-in roles.
         """
+        # role with node type specified, can only be edited from a
+        # node of the same type
+        if role.node_type and role.node_type != self.rattail_config.node_type():
+            return False
+
         if role is administrator_role(self.Session()):
             return False
         if role is authenticated_role(self.Session()):
@@ -142,16 +163,47 @@ class RoleView(PrincipalMasterView):
         super(RoleView, self).configure_form(f)
         role = f.model_instance
         use_buefy = self.get_use_buefy()
+        app = self.get_rattail_app()
+        auth = app.get_auth_handler()
 
         # name
         f.set_validator('name', self.unique_name)
 
+        # session_timeout
+        f.set_renderer('session_timeout', self.render_session_timeout)
+        if self.editing and role is guest_role(self.Session()):
+            f.set_readonly('session_timeout')
+
+        # sync_me, node_type
+        if not self.creating:
+            include = True
+            if role is administrator_role(self.Session()):
+                include = False
+            elif role is authenticated_role(self.Session()):
+                include = False
+            elif role is guest_role(self.Session()):
+                include = False
+            if not include:
+                f.remove('sync_me', 'sync_users', 'node_type')
+            else:
+                if not self.has_perm('edit_node_sync'):
+                    f.set_readonly('sync_me')
+                    f.set_readonly('sync_users')
+                    f.set_readonly('node_type')
+
         # notes
-        f.set_type('notes', 'text')
+        f.set_type('notes', 'text_wrapped')
+
+        # users
+        if use_buefy and self.viewing:
+            f.set_renderer('users', self.render_users)
+        else:
+            f.remove('users')
 
         # permissions
         self.tailbone_permissions = self.get_available_permissions()
-        f.set_renderer('permissions', PermissionsRenderer(permissions=self.tailbone_permissions))
+        f.set_renderer('permissions', PermissionsRenderer(request=self.request,
+                                                          permissions=self.tailbone_permissions))
         f.set_node('permissions', colander.Set())
         f.set_widget('permissions', PermissionsWidget(
             permissions=self.tailbone_permissions,
@@ -160,16 +212,47 @@ class RoleView(PrincipalMasterView):
             granted = []
             for groupkey in self.tailbone_permissions:
                 for key in self.tailbone_permissions[groupkey]['perms']:
-                    if has_permission(self.Session(), role, key, include_guest=False, include_authenticated=False):
+                    if auth.has_permission(self.Session(), role, key,
+                                           include_guest=False,
+                                           include_authenticated=False):
                         granted.append(key)
             f.set_default('permissions', granted)
         elif self.deleting:
             f.remove_field('permissions')
 
-        # session_timeout
-        f.set_renderer('session_timeout', self.render_session_timeout)
-        if self.editing and role is guest_role(self.Session()):
-            f.set_readonly('session_timeout')
+    def render_users(self, role, field):
+
+        if role is guest_role(self.Session()):
+            return ("The guest role is implied for all anonymous users, "
+                    "i.e. when not logged in.")
+
+        if role is authenticated_role(self.Session()):
+            return ("The authenticated role is implied for all users, "
+                    "but only when logged in.")
+
+        route_prefix = self.get_route_prefix()
+        permission_prefix = self.get_permission_prefix()
+        factory = self.get_grid_factory()
+        g = factory(
+            key='{}.users'.format(route_prefix),
+            data=[],
+            columns=[
+                'full_name',
+                'username',
+                'active',
+            ],
+            sortable=True,
+            sorters={'full_name': True, 'username': True, 'active': True},
+            default_sortkey='full_name',
+        )
+
+        if self.request.has_perm('users.view'):
+            g.main_actions.append(self.make_action('view', icon='eye'))
+        if self.request.has_perm('users.edit'):
+            g.main_actions.append(self.make_action('edit', icon='edit'))
+
+        return HTML.literal(
+            g.render_buefy_table_element(data_prop='usersData'))
 
     def get_available_permissions(self):
         """
@@ -237,13 +320,16 @@ class RoleView(PrincipalMasterView):
         permissions, but rather each "available" permission (depends on current
         user) will be examined individually, and updated as needed.
         """
+        app = self.get_rattail_app()
+        auth = app.get_auth_handler()
+
         available = self.tailbone_permissions
         for gkey, group in six.iteritems(available):
             for pkey, perm in six.iteritems(group['perms']):
                 if pkey in permissions:
-                    grant_permission(role, pkey)
+                    auth.grant_permission(role, pkey)
                 else:
-                    revoke_permission(role, pkey)
+                    auth.revoke_permission(role, pkey)
 
     def template_kwargs_view(self, **kwargs):
         role = kwargs['instance']
@@ -259,8 +345,28 @@ class RoleView(PrincipalMasterView):
                                          main_actions=actions)
         else:
             kwargs['users'] = None
+
         kwargs['guest_role'] = guest_role(self.Session())
         kwargs['authenticated_role'] = authenticated_role(self.Session())
+
+        use_buefy = self.get_use_buefy()
+        if use_buefy:
+            role = kwargs['instance']
+            if role not in (kwargs['guest_role'], kwargs['authenticated_role']):
+                users_data = []
+                for user in role.users:
+                    users_data.append({
+                        'uuid': user.uuid,
+                        'full_name': user.display_name,
+                        'username': user.username,
+                        'active': "Yes" if user.active else "No",
+                        '_action_url_view': self.request.route_url('users.view',
+                                                                   uuid=user.uuid),
+                        '_action_url_edit': self.request.route_url('users.edit',
+                                                                   uuid=user.uuid),
+                    })
+                kwargs['users_data'] = users_data
+
         return kwargs
 
     def before_delete(self, role):
@@ -272,13 +378,16 @@ class RoleView(PrincipalMasterView):
             return self.redirect(self.request.get_referrer(default=self.request.route_url('roles')))
 
     def find_principals_with_permission(self, session, permission):
+        app = self.get_rattail_app()
+        auth = app.get_auth_handler()
+
         # TODO: this should search Permission table instead, and work backward to Role?
         all_roles = session.query(model.Role)\
                            .order_by(model.Role.name)\
                            .options(orm.joinedload(model.Role._permissions))
         roles = []
         for role in all_roles:
-            if has_permission(session, role, permission, include_guest=False):
+            if auth.has_permission(session, role, permission, include_guest=False):
                 roles.append(role)
         return roles
 
@@ -287,6 +396,9 @@ class RoleView(PrincipalMasterView):
         View which renders the complete role / permissions matrix data into an
         Excel spreadsheet, and returns that file.
         """
+        app = self.get_rattail_app()
+        auth = app.get_auth_handler()
+
         roles = self.Session.query(model.Role)\
                             .order_by(model.Role.name)\
                             .all()
@@ -335,7 +447,8 @@ class RoleView(PrincipalMasterView):
 
                 # and show an 'X' for any role which has this perm
                 for col, role in enumerate(roles, 2):
-                    if has_permission(self.Session(), role, key, include_guest=False):
+                    if auth.has_permission(self.Session(), role, key,
+                                           include_guest=False):
                         sheet.cell(row=writing_row, column=col, value="X")
 
                 writing_row += 1
@@ -356,6 +469,7 @@ class RoleView(PrincipalMasterView):
         route_prefix = cls.get_route_prefix()
         url_prefix = cls.get_url_prefix()
         permission_prefix = cls.get_permission_prefix()
+        model_title = cls.get_model_title()
 
         # extra permissions for editing built-in roles etc.
         config.add_tailbone_permission(permission_prefix, '{}.edit_authenticated'.format(permission_prefix),
@@ -364,6 +478,9 @@ class RoleView(PrincipalMasterView):
                                        "Edit the \"Guest\" Role")
         config.add_tailbone_permission(permission_prefix, '{}.edit_my'.format(permission_prefix),
                                        "Edit Role(s) to which current user belongs")
+        config.add_tailbone_permission(permission_prefix,
+                                       '{}.edit_node_sync'.format(permission_prefix),
+                                       "Edit the Node Type and Sync flags for a {}".format(model_title))
 
         # download permissions matrix
         config.add_tailbone_permission(permission_prefix, '{}.download_permissions_matrix'.format(permission_prefix),
