@@ -43,6 +43,7 @@ class VendorView(MasterView):
     has_versions = True
     touchable = True
     supports_autocomplete = True
+    configurable = True
 
     labels = {
         'id': "ID",
@@ -168,6 +169,16 @@ class VendorView(MasterView):
             (model.VendorContact, 'vendor_uuid'),
         ]
 
+    def configure_get_simple_settings(self):
+        config = self.rattail_config
+        return [
+
+            # display
+            {'section': 'rattail',
+             'option': 'vendor.use_autocomplete',
+             'type': bool},
+        ]
+
 
 def includeme(config):
     VendorView.defaults(config)
